@@ -59,6 +59,7 @@ class SASconfigHTTP:
       client_id      = cfg.get('client_id', None)
       client_secret  = cfg.get('client_secret', '')
       authcode       = cfg.get('authcode', None)
+      jwt            = cfg.get('jwt', None)
       self.encoding  = cfg.get('encoding', '')
       self.authkey   = cfg.get('authkey', '')
       self._prompt   = session._sb.sascfg._prompt
@@ -154,17 +155,15 @@ class SASconfigHTTP:
 
       inautht = kwargs.get('authtoken', None)
       if inautht is not None:
-         if lock and self._token:
-            print("Parameter 'authtoken' passed to SAS_session was ignored due to configuration restriction.")
-         else:
-            self._token = inautht
+         self._token = inautht
+
+      injwt = kwargs.get('jwt', None)
+      if injwt is not None:
+         jwt = injwt
 
       inauthc = kwargs.get('authcode', None)
       if inauthc is not None:
-         if lock and authcode:
-            print("Parameter 'authcode' passed to SAS_session was ignored due to configuration restriction.")
-         else:
-            authcode = inauthc
+         authcode = inauthc
 
       incis = kwargs.get('client_secret', None)
       if incis is not None:
@@ -234,7 +233,7 @@ class SASconfigHTTP:
          else:
             self.port = 80
 
-      if not self._token and not authcode:
+      if not self._token and not authcode and not jwt:
          found = False
          if self.authkey:
             if os.name == 'nt':
@@ -319,21 +318,21 @@ class SASconfigHTTP:
             try:
                self.HTTPConn = hc.HTTPSConnection(self.ip, self.port, timeout=self.timeout)
                if not self._token:
-                  self._token = self._authenticate(user, pw, authcode, client_id, client_secret)
+                  self._token = self._authenticate(user, pw, authcode, client_id, client_secret, jwt)
             except ssl.SSLError as e:
                print("SSL certificate verification failed, creating an unverified SSL connection. Error was:"+str(e))
                self.HTTPConn = hc.HTTPSConnection(self.ip, self.port, timeout=self.timeout, context=ssl._create_unverified_context())
                print("You can set 'verify=False' to get rid of this message ")
                if not self._token:
-                  self._token   = self._authenticate(user, pw, authcode, client_id, client_secret)
+                  self._token   = self._authenticate(user, pw, authcode, client_id, client_secret, jwt)
          else:
             self.HTTPConn = hc.HTTPSConnection(self.ip, self.port, timeout=self.timeout, context=ssl._create_unverified_context())
             if not self._token:
-               self._token = self._authenticate(user, pw, authcode, client_id, client_secret)
+               self._token = self._authenticate(user, pw, authcode, client_id, client_secret, jwt)
       else:
          self.HTTPConn = hc.HTTPConnection(self.ip, self.port, timeout=self.timeout)
          if not self._token:
-            self._token   = self._authenticate(user, pw, authcode, client_id, client_secret)
+            self._token   = self._authenticate(user, pw, authcode, client_id, client_secret, jwt)
 
       if not self._token:
          print("Could not acquire an Authentication Token")
@@ -412,22 +411,30 @@ class SASconfigHTTP:
 
       return
 
-   def _authenticate(self, user, pw, authcode, client_id, client_secret):
+   def _authenticate(self, user, pw, authcode, client_id, client_secret, jwt):
       #import pdb; pdb.set_trace()
-      if authcode:
+      if   authcode:
          uauthcode      = urllib.parse.quote(authcode)
          uclient_id     = urllib.parse.quote(client_id)
          uclient_secret = urllib.parse.quote(client_secret)
-         d1 = ("grant_type=authorization_code&code="+uauthcode+"&client_id="+uclient_id+"&client_secret="+uclient_secret).encode(self.encoding)
-         headers={"Accept":"application/vnd.sas.compute.session+json","Content-Type":"application/x-www-form-urlencoded"}
+         d1             = ("grant_type=authorization_code&code="+uauthcode+
+                          "&client_id="+uclient_id+"&client_secret="+uclient_secret).encode(self.encoding)
+         headers        = {"Accept":"application/vnd.sas.compute.session+json","Content-Type":"application/x-www-form-urlencoded"}
+      elif jwt:
+         ujwt           = urllib.parse.quote(jwt)
+         d1             = "grant_type=urn:ietf:params:oauth:grant-type:jwt-bearer&assertion="+ujwt
+         client         = "Basic "+base64.encodebytes((client_id+":").encode(self.encoding)).splitlines()[0].decode(self.encoding)
+         headers        = {"Accept":"application/vnd.sas.compute.session+json",
+                           "Content-Type":"application/x-www-form-urlencoded",
+                           "Authorization":client}
       else:
-         uuser = urllib.parse.quote(user)
-         upw   = urllib.parse.quote(pw)
-         d1 = ("grant_type=password&username="+uuser+"&password="+upw).encode(self.encoding)
-         basic = base64.encodebytes("sas.tkmtrb:".encode(self.encoding))
-         authheader = '%s' % basic.splitlines()[0].decode(self.encoding)
-         headers={"Accept":"application/vnd.sas.compute.session+json","Content-Type":"application/x-www-form-urlencoded",
-                  "Authorization":"Basic "+authheader}
+         uuser          = urllib.parse.quote(user)
+         upw            = urllib.parse.quote(pw)
+         d1             = ("grant_type=password&username="+uuser+"&password="+upw).encode(self.encoding)
+         client         = "Basic "+base64.encodebytes("sas.tkmtrb:".encode(self.encoding)).splitlines()[0].decode(self.encoding)
+         #client        = "Basic "+base64.encodebytes((client_id+":").encode(self.encoding)).splitlines()[0].decode(self.encoding)
+         headers        = {"Accept":"application/vnd.sas.compute.session+json","Content-Type":"application/x-www-form-urlencoded",
+                           "Authorization":client}
 
       # POST AuthToken
       conn = self.HTTPConn; conn.connect()
@@ -728,6 +735,8 @@ class SASsessionHTTP():
       try:
          js = json.loads(resp.decode(self.sascfg.encoding))
          results = js.get('items')
+         if not results:
+            results = []
       except:
          results = []
 
@@ -994,6 +1003,7 @@ class SASsessionHTTP():
       Returns True it the Data Set exists and False if it does not
       '''
       #can't have an empty libref, so check for user or work
+      sd = table.strip().replace("'", "''")
       if not libref:
          # HEAD Libref USER
          conn = self.sascfg.HTTPConn; conn.connect()
@@ -1010,10 +1020,10 @@ class SASsessionHTTP():
 
       code  = 'data _null_; e = exist("'
       code += libref+"."
-      code += "'"+table.strip()+"'n"+'"'+");\n"
+      code += "'"+sd+"'n"+'"'+");\n"
       code += 'v = exist("'
       code += libref+"."
-      code += "'"+table.strip()+"'n"+'"'+", 'VIEW');\n if e or v then e = 1;\n"
+      code += "'"+sd+"'n"+'"'+", 'VIEW');\n if e or v then e = 1;\n"
       code += "te='TABLE_EXISTS='; put te e;run;\n"
 
       ll = self.submit(code, "text")
@@ -1058,7 +1068,7 @@ class SASsessionHTTP():
       code += "proc import datafile=x out="
       if len(libref):
          code += libref+"."
-      code += "'"+table.strip()+"'n dbms=csv replace; "+self._sb._impopts(opts)+" run;"
+      code += "'"+table.strip().replace("'", "''")+"'n dbms=csv replace; "+self._sb._impopts(opts)+" run;"
 
       if nosub:
          print(code)
@@ -1080,7 +1090,7 @@ class SASsessionHTTP():
       if len(libref):
          code += libref+"."
 
-      code += "'"+table.strip()+"'n "+self._sb._dsopts(dsopts)+" outfile=x dbms=csv replace; "
+      code += "'"+table.strip().replace("'", "''")+"'n "+self._sb._dsopts(dsopts)+" outfile=x dbms=csv replace; "
       code += self._sb._expopts(opts)+" run\n;"
       code += "options source;\n"
 
@@ -1319,8 +1329,8 @@ class SASsessionHTTP():
          warnings.warn("Note that Indexes are not transferred over as columns. Only actual coulmns are transferred")
 
       for name in df.columns:
-         colname = str(name)
-         col_up  = colname.upper()
+         colname = str(name).replace("'", "''")
+         col_up  = str(name).upper()
          input  += "'"+colname+"'n "
          if col_up in lab_keys:
             label += "label '"+colname+"'n ="+lab_upper[col_up]+";\n"
@@ -1371,7 +1381,7 @@ class SASsessionHTTP():
       code = "data "
       if len(libref):
          code += libref+"."
-      code += "'"+table.strip()+"'n"
+      code += "'"+table.strip().replace("'", "''")+"'n"
 
       if len(outdsopts):
          code += '('
@@ -1509,9 +1519,9 @@ class SASsessionHTTP():
       opts   = opts   if   opts is not None else {}
 
       if libref:
-         tabname = libref+".'"+table.strip()+"'n "
+         tabname = libref+".'"+table.strip().replace("'", "''")+"'n "
       else:
-         tabname = "'"+table.strip()+"'n "
+         tabname = "'"+table.strip().replace("'", "''")+"'n "
 
       code  = "data work.sasdata2dataframe / view=work.sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";run;\n"
 
@@ -1546,6 +1556,10 @@ class SASsessionHTTP():
          varlist.append(lst[i].get('name'))
          vartype.append(lst[i].get('type'))
 
+      dvarlist = list(varlist)
+      for i in range(len(varlist)):
+         varlist[i] = varlist[i].replace("'", "''")
+
       topts = dict(dsopts)
       topts.pop('firstobs', None)
       topts.pop('obs', None)
@@ -1597,11 +1611,11 @@ class SASsessionHTTP():
          for i in range(nvars):
             if vartype[i] == 'FLOAT':
                if varcat[i] not in self._sb.sas_date_fmts + self._sb.sas_time_fmts + self._sb.sas_datetime_fmts:
-                  dts[varlist[i]] = 'float'
+                  dts[dvarlist[i]] = 'float'
                else:
-                  dts[varlist[i]] = 'str'
+                  dts[dvarlist[i]] = 'str'
             else:
-               dts[varlist[i]] = 'str'
+               dts[dvarlist[i]] = 'str'
       else:
          dts = k_dts
 
@@ -1636,7 +1650,7 @@ class SASsessionHTTP():
          for i in range(nvars):
             if vartype[i] == 'FLOAT':
                if varcat[i] in self._sb.sas_date_fmts + self._sb.sas_time_fmts + self._sb.sas_datetime_fmts:
-                  df[varlist[i]] = pd.to_datetime(df[varlist[i]], errors='coerce')
+                  df[dvarlist[i]] = pd.to_datetime(df[dvarlist[i]], errors='coerce')
 
       ll = self.submit("filename _sp_updn;", 'text')
       logf += ll['LOG']
@@ -1670,9 +1684,9 @@ class SASsessionHTTP():
       dsopts = dsopts if dsopts is not None else {}
 
       if libref:
-         tabname = libref+".'"+table.strip()+"'n "
+         tabname = libref+".'"+table.strip().replace("'", "''")+"'n "
       else:
-         tabname = "'"+table.strip()+"'n "
+         tabname = "'"+table.strip().replace("'", "''")+"'n "
 
       code  = "data work.sasdata2dataframe / view=work.sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";run;\n"
 
@@ -1707,6 +1721,10 @@ class SASsessionHTTP():
          varlist.append(lst[i].get('name'))
          vartype.append(lst[i].get('type'))
 
+      dvarlist = list(varlist)
+      for i in range(len(varlist)):
+         varlist[i] = varlist[i].replace("'", "''")
+
       topts = dict(dsopts)
       topts.pop('firstobs', None)
       topts.pop('obs', None)
@@ -1770,10 +1788,10 @@ class SASsessionHTTP():
                         '%02x%02x' %                               \
                         (ord(rowsep.encode(self.sascfg.encoding)), \
                          ord(colsep.encode(self.sascfg.encoding))))
-            miss[varlist[i]] = ' '
+            miss[dvarlist[i]] = ' '
          else:
-            code += "if missing('"+varlist[i]+"'n) then '"+varlist[i]+"'n = '.'; "
-            miss[varlist[i]] = '.'
+            code += "if missing('"+varlist[i]+"'n) then '"+varlist[i]+"'n = .; "
+            miss[dvarlist[i]] = '.'
          if i % 10 == 9:
             code +='\n'
       code += "\nput "
@@ -1790,11 +1808,11 @@ class SASsessionHTTP():
          for i in range(nvars):
             if vartype[i] == 'FLOAT':
                if varcat[i] not in self._sb.sas_date_fmts + self._sb.sas_time_fmts + self._sb.sas_datetime_fmts:
-                  dts[varlist[i]] = 'float'
+                  dts[dvarlist[i]] = 'float'
                else:
-                  dts[varlist[i]] = 'str'
+                  dts[dvarlist[i]] = 'str'
             else:
-               dts[varlist[i]] = 'str'
+               dts[dvarlist[i]] = 'str'
       else:
          dts = k_dts
 
@@ -1816,7 +1834,7 @@ class SASsessionHTTP():
 
       sockout = _read_sock(req=req, method='DISK', rsep=(colsep+rowsep+'\n').encode(), rowsep=rowsep.encode())
 
-      df = pd.read_csv(sockout, index_col=idx_col, engine=eng, header=None, names=varlist,
+      df = pd.read_csv(sockout, index_col=idx_col, engine=eng, header=None, names=dvarlist,
                        sep=colsep, lineterminator=rowsep, dtype=dts, na_values=miss,
                        encoding='utf-8', quoting=quoting, **kwargs)
 
@@ -1826,7 +1844,7 @@ class SASsessionHTTP():
          for i in range(nvars):
             if vartype[i] == 'FLOAT':
                if varcat[i] in self._sb.sas_date_fmts + self._sb.sas_time_fmts + self._sb.sas_datetime_fmts:
-                  df[varlist[i]] = pd.to_datetime(df[varlist[i]], errors='coerce')
+                  df[dvarlist[i]] = pd.to_datetime(df[dvarlist[i]], errors='coerce')
 
       ll = self.submit("filename _sp_updn;", 'text')
       logf += ll['LOG']
